// Copyright (c) 2019, Christoph Peters
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Karlsruhe Institute of Technology nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "NearestNeighborIndex.h"
#include <iostream>

NearestNeighborIndex::NearestNeighborIndex(const PointList& rPointList, const Point& rPointToIgnore) :
	m_pointListAdaptor(m_reducedPointList),
	m_nearestNeighborIndex(3, m_pointListAdaptor, nanoflann::KDTreeSingleIndexAdaptorParams(16))
{
	// Count how many points are not ignored
	std::size_t relevantPointCount = 0;
	for (const Point& rPoint : rPointList) {
		if (rPoint[0] != rPointToIgnore[0] || rPoint[1] != rPointToIgnore[1] || rPoint[2] != rPointToIgnore[2]) {
			++relevantPointCount;
		}
	}
	std::cout << "The nearest neighbor index has " << relevantPointCount << " points." << std::endl;
	// Allocate space and fill the buffers
	m_reducedPointList.reserve(relevantPointCount);
	m_reducedPointListIndex.reserve(relevantPointCount);
	std::size_t pointIndex = 0;
	for (const Point& rPoint : rPointList) {
		if (rPoint[0] != rPointToIgnore[0] || rPoint[1] != rPointToIgnore[1] || rPoint[2] != rPointToIgnore[2]) {
			m_reducedPointList.push_back(rPoint);
			m_reducedPointListIndex.push_back(pointIndex);
		}
		++pointIndex;
	}
	// Build the nearest neighbor index
	m_nearestNeighborIndex.buildIndex();
}


std::size_t NearestNeighborIndex::query(const std::array<float, 3>& rQueryPoint)
{
	std::size_t reducedIndex;
	float squaredDistance;
	m_nearestNeighborIndex.knnSearch(&rQueryPoint[0], 1, &reducedIndex, &squaredDistance);
	return m_reducedPointListIndex[reducedIndex];
}
